﻿using Microsoft.Web.Administration;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.DirectoryServices;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using System.Web.Mvc;

namespace IISWebManager.Controllers
{
    public class HomeController : Controller
    {
        bool isAutoSetWebName = true;
        static DirectoryEntry iisDE = new DirectoryEntry("IIS://localhost/W3SVC");
        ServerManager iismanager = new ServerManager();
        public ActionResult Index()
        {
            //获取客户端电脑名
            IPHostEntry hostInfo = Dns.GetHostEntry(Request.UserHostAddress);
            ViewBag.LocalHostName = hostInfo.HostName;
            //验证权限
            var viewSiteNames = ConfigurationManager.AppSettings["PcNamesOfViewSite"];
            ViewBag.IsCanView = viewSiteNames != null && viewSiteNames != "" && viewSiteNames.Contains(hostInfo.HostName.ToLower());
            return View();
        }

        #region 获取本机的计算机名
        /// <summary>
        /// 获取本机的计算机名
        /// </summary>
        public static string LocalHostName
        {
            get
            {
                return Dns.GetHostName();
            }
        }
        #endregion

        public async Task<JsonResult> InfoList(string siteNameKey = "")
        {
            var data = await GetSiteInfos(siteNameKey);
            return Json(new { Data = data, Count = data.Count });
        }

        //需要将当前站点的应用池标记（identity）改为LocalSystem
        protected async Task<List<SiteInfo>> GetSiteInfos(string siteNameKey)
        {
            //获取iis全部的网站名称和对应的url地址
            var webs = new List<SiteInfo>();
            DirectoryEntry rootEntry = new DirectoryEntry("IIS://localhost/w3svc");
            foreach (DirectoryEntry entry in rootEntry.Children)
            {
                if (entry.SchemaClassName.Equals("IIsWebServer", StringComparison.OrdinalIgnoreCase))
                {
                    var web = new SiteInfo();
                    web.name = entry.Properties["ServerComment"].Value.ToString();//网站名称                   
                    web.path = GetWebsitePhysicalPath(entry);//网站物理路径
                    web.port = $"{entry.Properties["ServerBindings"].Value}".Trim(':');//网站端口
                    try
                    {
                        var appPool = iismanager.ApplicationPools[web.name];
                        if (appPool == null)
                        {
                            appPool = iismanager.ApplicationPools[web.port];
                        }
                        var runtimeVersion = appPool?.ManagedRuntimeVersion.ToString();
                        web.clrVersion = runtimeVersion.StartsWith("v") ? runtimeVersion : "无托管代码";
                        web.appPoolState = appPool?.State.ToString();
                        web.clrMode = appPool?.ManagedPipelineMode.ToString();
                    }
                    catch
                    {

                    }
                    if (!string.IsNullOrEmpty(siteNameKey) && !web.name.Contains(siteNameKey))
                    {
                        continue;
                    }
                    webs.Add(web);
                }
            }
            return await Task.FromResult(webs.Where(e => !e.name.Contains("IISWebManager")).OrderBy(e => e.name).ToList());
        }

        /// <summary>
        /// 得到网站的物理路径
        /// </summary>
        /// <param name="rootEntry">网站节点</param>
        /// <returns></returns>
        public string GetWebsitePhysicalPath(DirectoryEntry rootEntry)
        {
            string physicalPath = "";
            foreach (DirectoryEntry childEntry in rootEntry.Children)
            {
                if ((childEntry.SchemaClassName == "IIsWebVirtualDir") && (childEntry.Name.ToLower() == "root"))
                {
                    if (childEntry.Properties["Path"].Value != null)
                    {
                        physicalPath = childEntry.Properties["Path"].Value.ToString();
                    }
                    else
                    {
                        physicalPath = "";
                    }
                }
            }
            return physicalPath;
        }
        public class SiteInfo
        {
            public string name { get; set; }
            public string port { get; set; }
            public string clrMode { get; set; }
            public string clrVersion { get; set; }
            public string path { get; set; }
            public string appPoolState { get; set; }
        }

        //发布站点
        public JsonResult PublishWeb(string webUrl, string port)
        {
            var r = "";
            try
            {
                if (port == "9999")
                {
                    return Json(new { state = false, message = "9999为系统内置端口，请勿使用！" });
                }
                string webName = webUrl.Substring(webUrl.LastIndexOf(@"\") + 1);
                webName = port + "-" + webName;

                ServerManager iismanager = new ServerManager();
                //判断应用程序池是否存在
                if (iismanager.ApplicationPools[webName] != null)
                {
                    iismanager.ApplicationPools.Remove(iismanager.ApplicationPools[webName]);
                }
                //判断web应用程序是否存在
                if (iismanager.Sites[webName] != null)
                {
                    iismanager.Sites.Remove(iismanager.Sites[webName]);
                }
                //建立web应用程序（第二个参数为安装文件的地址）
                iismanager.Sites.Add(webName, webUrl, int.Parse(port));
                //添加web应用程序池
                ApplicationPool pool = iismanager.ApplicationPools.Add(webName);
                //设置web应用程序池的Framework版本（注意版本号大小写问题）
                pool.ManagedRuntimeVersion = "无托管代码";//v4.0
                //设置是否启用32为应用程序
                pool.SetAttributeValue("enable32BitAppOnWin64", true);
                //设置web网站的应用程序池
                iismanager.Sites[webName].Applications[0].ApplicationPoolName = webName;
                //提交更改
                iismanager.CommitChanges();
                r = "your site is added success~";
            }
            catch (Exception ex)
            {
                //throw (ex);
                r = ex.Message;
                return Json(new { state = false, message = r });
            }
            return Json(new { state = true, message = r });
        }

        //移出站点
        public JsonResult RemoveSite(string webName, string port)
        {
            //判断应用程序池是否存在
            if (iismanager.ApplicationPools[webName] != null)
            {
                iismanager.ApplicationPools.Remove(iismanager.ApplicationPools[webName]);
            }
            else if (iismanager.ApplicationPools[port] != null)
            {
                iismanager.ApplicationPools.Remove(iismanager.ApplicationPools[port]);
            }
            else
            {
                return Json(new { state = false, message = "your site's appPool is not exist！" });
            }
            //判断web应用程序是否存在
            if (iismanager.Sites[webName] != null)
            {
                iismanager.Sites.Remove(iismanager.Sites[webName]);
            }
            else if (iismanager.Sites[port] != null)
            {
                iismanager.Sites.Remove(iismanager.Sites[port]);
            }
            else
            {
                return Json(new { state = false, message = "your site is not exist！" });
            }
            //提交更改
            iismanager.CommitChanges();

            return Json(new { state = true, message = "operate success！" });
        }

        //切换clr版本
        public JsonResult SwitchSiteCLR_Version(string webName, string port,string clrVersion)
        {
            //判断应用程序池是否存在
            if (iismanager.ApplicationPools[webName] != null)
            {
                iismanager.ApplicationPools[webName].ManagedRuntimeVersion = clrVersion;
            }
            else if (iismanager.ApplicationPools[port] != null)
            {
                iismanager.ApplicationPools[port].ManagedRuntimeVersion = clrVersion;
            }
            else
            {
                return Json(new { state = false, message = "your site's appPool is not exist！" });
            }
            //提交更改
            iismanager.CommitChanges();

            return Json(new { state = true, message = "operate success！" });
        }
        
        //切换clr模式
        public JsonResult SwitchSiteCLR_Model(string webName, string port, string clrMode)
        {
            var clrModeName = clrMode == "Classic" ? ManagedPipelineMode.Classic : ManagedPipelineMode.Integrated;
            //判断应用程序池是否存在
            if (iismanager.ApplicationPools[webName] != null)
            {
                iismanager.ApplicationPools[webName].ManagedPipelineMode = clrModeName;
            }
            else if (iismanager.ApplicationPools[port] != null)
            {
                iismanager.ApplicationPools[port].ManagedPipelineMode = clrModeName;
            }
            else
            {
                return Json(new { state = false, message = "your site's appPool is not exist！" });
            }
            //提交更改
            iismanager.CommitChanges();

            return Json(new { state = true, message = "operate success！" });
        }

        //启动or停止站点应用程序池
        public JsonResult StartOrStopSite(string webName, string port, bool startOrStop)
        {
            var msg = "";
            if (startOrStop)
            {
                msg = StopAppPool(webName, port);
            }
            else
            {
                msg = StartAppPool(webName, port);
            }
            return Json(new { state = true, message = msg });
        }

        #region 网站应用程序池启动回收停止
        private string StartAppPool(string webName, string port)
        {
            try
            {
                //如果应用程序池不存在,则会报错系统找不到指定路径
                string method = "Start";

                DirectoryEntry appPool = new DirectoryEntry("IIS://localhost/W3SVC/AppPools");
                DirectoryEntry findPool = null;
                if (iismanager.ApplicationPools[webName] != null)
                {
                    findPool = appPool.Children.Find(webName, "IIsApplicationPool");
                }
                else
                {
                    findPool = appPool.Children.Find(port, "IIsApplicationPool");
                }
                findPool.Invoke(method, null);
                appPool.CommitChanges();
                appPool.Close();
                return "operate success！";
            }
            catch (Exception ex)
            {
                return ex.Message;
            }
        }

        private void RecycleAppPool(string AppPoolName)
        {
            //如果应用程序池当前状态为停止,则会发生异常报错
            try
            {
                string method = "Recycle";

                DirectoryEntry appPool = new DirectoryEntry("IIS://localhost/W3SVC/AppPools");
                DirectoryEntry findPool = appPool.Children.Find(AppPoolName, "IIsApplicationPool");
                findPool.Invoke(method, null);
                appPool.CommitChanges();
                appPool.Close();
            }
            catch (Exception ex)
            {
                //MessageBox.Show(ex.Message, "回收失败");
            }
        }

        private string StopAppPool(string webName, string port)
        {
            try
            {
                string method = "Stop";

                DirectoryEntry appPool = new DirectoryEntry("IIS://localhost/W3SVC/AppPools");
                DirectoryEntry findPool = null;
                if (iismanager.ApplicationPools[webName] != null)
                {
                    findPool = appPool.Children.Find(webName, "IIsApplicationPool");
                }
                else
                {
                    findPool = appPool.Children.Find(port, "IIsApplicationPool");
                }
                findPool.Invoke(method, null);
                appPool.CommitChanges();
                appPool.Close();
                return "operate success！";
            }
            catch (Exception ex)
            {
                return ex.Message;
            }
        }
        #endregion


    }
}